import json

from portal.database.models import Component
from tests.rest import utils


def test_list(client):
    response = utils.make_request(client, '/components', method='get')
    assert response.status_code == 200
    assert response.mimetype == 'application/json'

    components = utils.extract_data(response)
    assert len(components) == 2
    db_components = Component.query.all()
    utils.assert_component_in(db_components, components[0])
    utils.assert_component_in(db_components, components[1])


def test_create(client):
    request_dict = dict(
        name="new_executor",
        ipaddr="127.0.0.1",
        secret='new_exec_secret',
        type='executor',
    )
    db_components_number = len(Component.query.all())

    request_json = json.dumps(request_dict)
    response = utils.make_request(client, '/components',
                                  data=request_json,
                                  headers={"content-type": "application/json"},
                                  method='post')
    assert response.status_code == 201
    assert response.mimetype == 'application/json'

    resp_new_component = utils.extract_data(response)
    db_components = Component.query.all()
    assert len(db_components) == db_components_number + 1
    utils.assert_component_in(db_components, resp_new_component)


def test_read(client):
    c = Component.query.filter_by(name="executor").first()
    response = utils.make_request(client, f"/components/{c.id}", method='get')
    assert response.status_code == 200
    assert response.mimetype == 'application/json'

    component = utils.extract_data(response)

    utils.assert_component(c, component)


def test_update(client):
    c = Component.query.filter_by(name="executor").first()
    request_dict = dict(
        notes="Super executor",
    )
    request_json = json.dumps(request_dict)
    response = utils.make_request(client, f'/components/{c.id}', data=request_json,
                                  headers={"content-type": "application/json"}, method='put')

    assert response.status_code == 204
    assert response.mimetype == 'application/json'

    component_after_update: Component = Component.query.filter_by(
        name="executor").first()
    assert component_after_update.notes == "Super executor"


def test_delete(client):
    c = Component.query.filter_by(name="executor").first()
    response = utils.make_request(client, f'/components/{c.id}',
                                  headers={"content-type": "application/json"},
                                  method='delete')

    assert response.status_code == 204
    assert response.mimetype == 'application/json'

    assert not Component.query.filter_by(name="executor").first()
